c 1 GEOUTM
c   Subroutine to convert between geodetic and utm.
c 2 USAGE
c   "a" = argument, "r" = referenced, "s" = set
c   
c   call geoutm                           (with the following arguments)
c          itype                       [variable i*4 r]
c          DoubleLatitude              [variable r*8 sr]
c          DoubleLongitude             [variable r*8 sr]
c          UTMX                        [variable r*8 a]
c          UTMY                        [variable r*8 a]
c          Zone                        [variable i*4 sr]
c    
c   itype = 1 if lat/long --> UTM
c   itype = 2 if utm --> lat/long
c 2 SUBROUTINES
c   tmcof               (internal)                    
c   tmfwd               (internal)                    
c   tminv               (internal)                    
c 2 FUNCTIONS
c   idint               (FORTRAN intrinsic function)  
c   iabs                (FORTRAN intrinsic function)  
c 2 DISCLAIMER
c   Although this program has been used by the U.S. Geological Survey,
c   no warranty, expressed or implied, is made by the Survey as to the
c   accuracy and functioning of the program and related program
c   material nor shall the fact of distribution constitute any such
c   warranty, and no responsibility is assured by the Survey in
c   connection therewith.
c 2 MODIFICATIONS
c   Added implicit none.    - Carolyn Degnan, 9/23/04

      subroutine geoutm ( itype, DoubleLatitude, DoubleLongitude, 
     *                    UTMX, UTMY, Zone )

      implicit none

      real * 8 slat, 
     *         slon, 
     *         sk, 
     *         thet, 
     *         a(16), 
     *         b(4)

      integer * 4 itype, 
     *            idlon, 
     *            ierr

      real * 4 utz

      real * 8 DoubleLatitude, 
     *         DoubleLongitude, 
     *         UTMX, 
     *         UTMY

      integer * 4 Zone 

c -  -  - INITIALIZE VARIABLES

      idlon = idint( DoubleLongitude )

      a(5) = 5.0d5
      a(6) = 0.0d0
      if ( DoubleLatitude .lt. 0.0d0 ) a(6) = 10.0d6
      if ( Zone .lt. 0 ) a(6) = 10.0d6
      a(7) = 0.0d0
      a(8) = 0.9996d0

c  COMPUTE SEMI - MAJOR & SEMI - MINOR AXES FOR UTM SPHEROID TO BE USED
c       SPHEROID BASED ON CLARKE 1866, NAD27

      a(15) = 6378206.4d0
      b(1)  = 6356583.8d0

      a(16) = ((a(15) - b(1)) / a(15)) * 
     *        ((a(15) + b(1)) / a(15))

c     COMPUTE COEFFICIENTS FOR CONVERSIONS

      call tmcof(a)

c     TEST FOR TYPE OF CONVERSION, GEODETIC TO UTM OR UTM TO GEODETIC.

      if (itype .eq. 1) then

c -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - 
c -  -  - GEODETIC TO UTM CONVERSION
c -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - 

          if ( Zone .ne. 0 ) then

c -  -  - COMPUTE CENTRAL MERIDIAN IN SECONDS WHERE UTM ZONE IS INPUTED

              if (( iabs(Zone) - 30 ) .le. 0) then
                  utz = 30.0d0 - iabs(Zone)
                  a(9) = ((utz * 6.0d0) + 3.0d0) * 3600.0d0
              else
                  utz = iabs(Zone) - 30.0d0
                  a(9) = ((utz * 6.0d0) - 3.0d0) * ( - 3600.0d0)
              end if

c    COMPUTE UTM ZONE(Zone) AND CENTRAL MERIDIAN IN SECONDS(A9) FOR
c     GEODETIC TO UTM CONVERSION WHERE ZONE IS NOT INPUT.

          else
              if ( DoubleLongitude .ge. 0.0d0 ) then
                  Zone = ( idlon / 6 ) + 31
              else
                  Zone = ( idlon / 6 ) + 30
              end if

              utz = 30.0d0 - Zone
              a(9) = ((utz * 6.0d0) + 3.0d0) * 3600.0d0

c     CONVERT LATITUDE AND LONGITUDE TO SECONDS

          end if

          slat =  DoubleLatitude *  3600.0d0
          slon =  DoubleLongitude * -3600.0d0

c     CONVERT GEODETIC TO UTM COORDINATES

          call tmfwd (slat, slon, UTMY , UTMX, a, ierr, sk, thet)

c -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - 
c -  -  - UTM TO GEODETIC CONVERSION
c -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - 

      else
          utz = 30.0d0 - iabs(Zone)
          a(9) = ((utz * 6.0d0) + 3.0d0) * 3600.0d0

c     CONVERT UTM COORDINATES TO GEODETIC

          call tminv (UTMY , UTMX, slat, slon, a, ierr, sk, thet)

c -  -  - CONVERT SECONDS TO DECIMAL DEGREES

          DoubleLatitude  =   2.77777777778d-4 * slat
          DoubleLongitude =  -2.77777777778d-4 * slon
      end if

c     ASSURE NEGATIVE ZONES WHEN IN SOUTHERN HEMISPHERE

      if (DoubleLatitude .lt. 0) then
          Zone = - iabs(Zone)
      end if

c      print *, 'Lat = ', DoubleLatitude
c      print *, 'Lon = ', DoubleLongitude
c      print *, 'UTMX = ', UTMX
c      print *, 'UTMY = ', UTMY
c      print *, 'Zone = ', Zone
c      print *, ' ' 
      return
      end


c 1 TMCOF
c   Subroutine to set up coefficients for converting geodetic to 
c     rectifying latitude and conversely.
c     
c 2 USAGE
c   call tmcof ( a )
c     
c 2 USAGE
c   "a" = argument, "r" = referenced, "s" = set
c   
c   call tmcof                            (with the following arguments)
c          a                           [array r*8 sr]
c           
c 2 DISCLAIMER
c   Although this program has been used by the U.S. Geological Survey,
c   no warranty, expressed or implied, is made by the Survey as to the
c   accuracy and functioning of the program and related program
c   material nor shall the fact of distribution constitute any such
c   warranty, and no responsibility is assured by the Survey in
c   connection therewith.
c 2 MODIFICATIONS
c   Added implicit none.    - Carolyn Degnan, 9/23/04

      subroutine tmcof ( a )

      implicit none

      real * 8 a(16), 
     *         fac

      a(10) = (((a(16) * (7.0d0 / 3.2d1) + (5.0d0 / 1.6d1)) * 
     *           a(16) + 0.5d0) * a(16) + 1.0d0) * a(16) * 0.25d0
      a(1)  = -(((a(10) * (1.95d2 / 6.4d1) + 3.25d0) * 
     *            a(10) + 3.75d0) * a(10) + 3.0d0) * a(10)
      a(2)  = (((1.455d3 / 3.2d1) * a(10) + (7.0d1 / 3.0d0)) * 
     *           a(10) + 7.5d0) * a(10)**2
      a(3)  = -((7.0d1 / 3.0d0) + a(10) * (9.45d2 / 8.0d0)) * 
     *           a(10)**3
      a(4)  = (3.15d2 / 4.0d0) * a(10)**4
      a(11) = (((7.75d0 - (6.57d2 / 6.4d1) * a(10)) * a(10) - 
     *         5.25d0) * a(10) + 3.0d0) * a(10)
      a(12) = (((5.045d3 / 3.2d1) * a(10) - (1.51d2 / 3.0d0)) * 
     *         a(10) + 10.5d0) * a(10)**2
      a(13) = ((1.51d2 / 3.0d0) - (3.291d3 / 8.0d0) * 
     *         a(10)) * a(10)**3
      a(14) = (1.097d3 / 4.0d0) * a(10)**4

c     A(1) TO A(4) ARE FOR GEODETIC TO RECTIFYING LATITUDE
c        CONVERSION WHILE A(11) TO A(14) ARE COEFFICIENTS FOR
c        RECTIFYING TO GEODETIC CONVERSION.

      fac    = a(10) * a(10)
      a(10)  = (((2.25d2 / 6.4d1) * fac + 2.25d0) * fac + 1.0d0) *
     *         (1.0d0 - fac) * (1.0d0 - a(10)) * a(15)

c     A(10) IS NOW SET TO RADIUS OF SPHERE WITH GREAT CIRCLE LENGTH
c        EQUAL TO SPHEROID MERIDIAN LENGTH.

      return
      end


c 1 TMFWD
c   Subroutine to convert lat/lon seconds to transverse Mercator (x,y).
c     
c 2 USAGE
c   "a" = argument, "r" = referenced, "s" = set
c   
c   call tmfwd                            (with the following arguments)
c      slat                                [variable r*8 r]
c      slon                                [variable r*8 r]
c      north                               [variable r*8 sr]
c      east                                [variable r*8 s]
c      a                                   [array r*8 r]
c      ierr                                [variable i*4 s]
c      sk                                  [variable r*8 sr]
c      thet                                [variable r*8 s]
c     
c   where:
c   slat = latitude  in seconds
c   slon = longitude in seconds
c   a(1) to a(4) are coefficients used
c        to convert geodetic latitude to rectifying latitude,
c   a(5) is false easting,
c   a(6) is false northing,
c   a(8) is scale factor at central meridian.
c   a(9) is central meridian in seconds.
c   a(10) is radius of sphere having a great circle length equal
c         to spheroid meridian length.
c   a(11) to a(14) are coefficients to convert rectifying latitude
c                  to geodetic latitude.
c   a(15) is semimajor axis of spheroid,  and
c   a(16) is eccentricity squared.
c   ierr is set to 1
c                    if lat exceeds 84 degrees, or
c                    if long exceeds 0.16 radians
c     
c 2 FUNCTIONS
c   dabs                (FORTRAN intrinsic function)  
c   dsin                (FORTRAN intrinsic function)  
c   dcos                (FORTRAN intrinsic function)  
c   dsqrt               (FORTRAN intrinsic function)  
c           
c 2 DISCLAIMER
c   Although this program has been used by the U.S. Geological Survey,
c   no warranty, expressed or implied, is made by the Survey as to the
c   accuracy and functioning of the program and related program
c   material nor shall the fact of distribution constitute any such
c   warranty, and no responsibility is assured by the Survey in
c   connection therewith.
c 2 MODIFICATIONS
c   Added implicit none.    - Carolyn Degnan, 9/23/04

      subroutine tmfwd(slat, slon, north, east, a, ierr, sk, thet)

      implicit none

      real * 8 slat, 
     *         slon, 
     *         a(16), 
     *         b(12), 
     *         sinp, 
     *         cosp, 
     *         rn, 
     *         t,
     *         ts, 
     *         etas, 
     *         north, 
     *         east, 
     *         sk, 
     *         thet

      integer * 4 ierr

      ierr = 0
      if ((dabs(slat)-302400.0d0) .gt. 0.0) then
          ierr  = 1
          east  = 0.0d0
          north = 0.0d0
          return
      end if

      b(10) = (a(9)-slon)  * 4.84813681109536d-6
      if ((dabs(b(10))-0.16d0) .gt. 0.0) then 
          ierr  = 1
          east  = 0.0d0
          north = 0.0d0
          return
      end if

      b(9)  = slat * 4.84813681109536d-6
      sinp  = dsin(b(9))
      cosp  = dcos(b(9))
      rn    = a(15) / dsqrt(1.0d0 - a(16) * sinp * sinp)
      t     = sinp / cosp
      ts    = t * t
      b(11) = cosp * cosp
      etas  = a(16) * b(11) / (1.0d0 - a(16))
      b(1)  = rn * cosp
      b(3)  = (1.0d0 - ts + etas) * b(1) * b(11) / 6.0d0
      b(5)  = ((ts - 18.0d0) * ts + 5.0d0 +
     *         (14.0d0 - 58.0d0 * ts) * etas) * b(1) * b(11) * 
     *         b(11) / 120.0d0
      b(7)  = (((179.0d0 - ts) * ts - 479.0d0) * ts + 61.0d0) * 
     *         b(1) * b(11)**3 / 5040.0d0
      b(12) = b(10) * b(10)
      east  = (((b(7) * b(12) + b(5)) * b(12) + b(3)) * b(12) + 
     *        b(1)) * b(10) * a(8) + a(5)
      b(2)  = rn * b(11) * t / 2.0d0
      b(4)  = (etas * (9.0d0 + 4.0d0 * etas) + 5.0d0 - ts) * 
     *        b(2) * b(11) / 12.0d0
      b(6)  = ((ts - 58.0d0) * ts + 61.0d0 + 
     *        (270.0d0 - 330.0d0 * ts) * etas) * b(2) * 
     *         b(11) * b(11) / 360.0d0
      b(8)  = (((543.0d0 - ts) * ts - 3111.0d0) * ts + 1385.0d0) *
     *         b(2) * b(11)**3 / 20160.0d0
      north = (((b(8) * b(12) + b(6)) * b(12) + b(4)) * b(12) + 
     *         b(2)) * b(12) + 
     *        ((((a(4) * b(11) + a(3)) * b(11) + a(2)) * b(11) + 
     *         a(1)) * sinp * cosp + b(9)) * a(10)
      north = (north - a(7)) * a(8) + a(6)

c     COMPUTE SCALE FACTOR  SK

      sk    = (((((-24.0d0 * etas - 48.0d0) * etas - 28.0d0) * 
     *        etas - 4.0d0) * ts) + 
     *        (((4.0d0 * etas + 13.0d0) * etas + 14.0d0) * 
     *        etas + 5.0d0)) * ((b(10)**4) / 24.0d0) * 
     *        b(11) * b(11) + (1.0d0 + etas) * b(11) * 
     *        (b(10) * b(10)) / 2.0d0 + 1.0d0
      sk    = sk * a(8)

c     COMPUTE CONVERGENCE ANGLE--THET

      thet  = (b(10) * sinp * (1.0d0 + ((b(10)**2) * 
     *         b(11) / 3.0d0) * (1.0d0 + 3.0d0 * 
     *         etas + 2.0d0 * etas**2) + (b(10)**4) * 
     *        ((b(11)**2) / 15.0d0) * (2.0d0-ts))) * 
     *         206264.8062470964d0

      return
      end


c 1 TMINV
c   Subroutine to compute lat/lon seconds from transverse Mercator (x,y).
c     
c 2 USAGE
c   "a" = argument, "r" = referenced, "s" = set
c   
c   call tminv                            (with the following arguments)
c          north                       [variable r*8 r]
c          east                        [variable r*8 r]
c          slat                        [variable r*8 s]
c          slon                        [variable r*8 s]
c          a                           [array r*8 r]
c          ierr                        [variable i*4 s]
c          sk                          [variable r*8 sr]
c          thet                        [variable r*8 sr]
c   where:
c   slat = latitude  in seconds
c   slon = longitude in seconds
c   a is array of parameters used in computation,
c        described by comments for tmfwd subroutine.
c   ierr set to 1 if grid distance from central meridian
c        exceeds 0.2 of spheroid semimajor axis numerically or if
c        if absolute value of rectifying latitude exceeds 1.47 radians.
c   south latitudes and east longitude are negative.
c     
c 2 FUNCTIONS
c   dabs                (FORTRAN intrinsic function)  
c   dsin                (FORTRAN intrinsic function)  
c   dcos                (FORTRAN intrinsic function)  
c   dsqrt               (FORTRAN intrinsic function)  
c           
c 2 DISCLAIMER
c   Although this program has been used by the U.S. Geological Survey,
c   no warranty, expressed or implied, is made by the Survey as to the
c   accuracy and functioning of the program and related program
c   material nor shall the fact of distribution constitute any such
c   warranty, and no responsibility is assured by the Survey in
c   connection therewith.
c 2 MODIFICATIONS
c   Added implicit none.    - Carolyn Degnan, 9/23/04

      subroutine tminv (north, east, slat, slon, a, ierr, sk, thet)

      implicit none

      real * 8 slat, 
     *         slon, 
     *         a(16), 
     *         b(12), 
     *         sinw, 
     *         cosw, 
     *         rn, 
     *         t,
     *         ts, 
     *         etas, 
     *         north, 
     *         east, 
     *         x, 
     *         y, 
     *         sk, 
     *         bn, 
     *         bns, 
     *         thet

      integer * 4 ierr

      y    = north
      x    = east
      ierr = 0
      b(9) = ((a(5) - x) * 1.0d-6) / a(8)

      if ((dabs(b(9)) - 1.0d-7 * a(15) * 2.0d0) .gt. 0.0) then
          ierr = 1
          slat = 0.0
          slon = 0.0
          return
      end if

      b(10) = ((y - a(6)) / a(8) + a(7)) / a(10)

      if ((dabs(b(10)) - 1.47d0) .gt. 0.0) then
          ierr = 1
          slat = 0.0
          slon = 0.0
          return
      end if

      sinw  = dsin(b(10))
      cosw  = dcos(b(10))
      b(12) = cosw * cosw
      b(11) = (((a(14) * b(12) + a(13)) * 
     *           b(12) + a(12)) * b(12) + a(11)) *
     *           sinw * cosw + b(10)
      sinw  = dsin(b(11))
      cosw  = dcos(b(11))
      rn    = dsqrt(1.0d0 - a(16) * sinw * sinw) * 1.0d6 / a(15)
      t     = sinw / cosw
      ts    = t * t
      b(12) = cosw * cosw
      etas  = a(16) * b(12) / (1.0d0 - a(16))
      b(1)  = rn / cosw
      b(2)  = -t * (1.0d0 + etas) * rn * rn / 2.0d0
      b(3)  = -(1.0d0 + 2.0d0 * ts + etas) * b(1) * rn * rn / 6.0d0
      b(4)  = (((-6.0d0 - etas * 9.0d0) * etas + 3.0d0) * ts + 
     *           (6.0d0 - etas * 3.0d0) * etas + 5.0d0) * t * 
     *            rn**4  / 24.0d0
      b(5)  = ((ts * 24.0d0 + etas * 8.0d0 + 28.0d0) * ts + 
     *          etas * 6.0d0 + 5.0d0) * b(1) * rn**4 / 120.0d0
      b(6)  = (((etas * 45.0d0 - 45.0d0) * ts + 
     *           etas * 162.0d0 - 90.0d0) * ts - 
     *           etas * 107.0d0 - 61.0d0) * t * rn**6 / 720.0d0
      b(7)  = -(((ts * 720.0d0 + 1320.0d0) * ts + 662.0d0) * ts + 
     *            61.0d0) * b(1) * rn**6 / 5040.0d0
      b(8)  = (((ts * 1575.0d0 + 4095.0d0) * ts + 3633.0d0) * ts + 
     *           1385.0d0) * t * rn**8 / 40320.0d0
      b(10) = b(9) * b(9)
      slat  = ((((b(8) * b(10) + b(6)) * 
     *            b(10) + b(4)) * b(10) + b(2)) * b(10) + b(11)) * 
     *            206264.8062470964d0
      slon  =  (((b(7) * b(10) + b(5)) * 
     *            b(10) + b(3)) * b(10) + b(1)) * b(9) * 
     *            206264.8062470964d0  +  a(9)
      bn    = b(9) * rn
      bns   = bn**2
      sk    = 1.0d0 + ((1.0d0 + etas) / 2.0d0) * bns + 
     *       (1.0d0 + 6.0d0 * etas + 
     *        9.0d0 * etas * etas + 
     *        4.0d0 * etas * etas * etas - 
     *       24.0d0 * etas * etas * ts -
     *       24.0d0 * etas * etas * etas * ts) * bns * bns / 
     *       24.0d0 + (bns * bns * bns) / 720.0d0
      sk    = sk * a(8)

c   COMPUTE CONVERGENCE ANGLE--THET

      thet  = (((((((-24.0d0 * etas - 27.0d0) * etas - 7.0d0) * 
     *        etas + 1.0d0) * etas) * ts) + 
     *        (5.0d0 * ts + 3.0d0 * ts**2) +
     *        ((((11.0d0 * etas + 20.0d0) * etas + 9.0d0) * 
     *        etas + 2.0d0) * etas) + 2.0d0) * 
     *        (bn**5) * t / 15.0d0) + (t * bn) - 
     *        (((45.0d0 * ts + 105.0d0) * ts + 77.0d0) * ts + 
     *        17.0d0) * ((bn**7) * t / 315.0d0) - (((1.0d0 + ts -
     *        etas - 2.0d0 * etas**2) * bn**3) * t / 3.0d0)

      thet  = -thet * 206264.8062470964d0

      return
      end
