c 1 TYPENAV
c   Subroutine to determine the type of navigation data 
c   (lat/long, UTM, unknown) and assure hemispherical consistancy
c   between two pairs of positions.
c 2 USAGE
c   "a" = argument, "r" = referenced, "s" = set
c   
c   call typenav                          (with the following arguments)
c          Input1x                     [variable r*8 r]
c          Input1y                     [variable r*8 r]
c          Input2x                     [variable r*8 r]
c          Input2y                     [variable r*8 r]
c          Output1x                    [variable r*8 sr]
c          Output1y                    [variable r*8 sr]
c          Output2x                    [variable r*8 sr]
c          Output2y                    [variable r*8 sr]
c          DataType                    [variable char*8 s]
c 2 FUNCTIONS
c   abs                 (FORTRAN intrinsic function)  
c 2 FILES_USED
c   unit=*(write)
c 2 LIMITATIONS
c   Positions can be either UTMx/UTMy pairs or Lat/Long pairs.
c   One position CAN NOT be UTMs with the other being Lat/Long.
c   Both UTM pairs are assumed to be based on the same UTM zone.
c   UTM positions within 360 meters of 0d lat and 0d long will be
c       interpreted as lat/longs not UTMs.
c   Lat/Longs may be on opposite sides of the International Date Line.
c   Longitudes must be between +/- 180.0
c   Identical input pairs results in a distance value of 0.0
c 2 AUTHOR
c   Clint Steele       3/16/93
c   Carolyn Degnan     9/23/04    added implicit none.

      subroutine typenav (Input1x, Input1y, Input2x, Input2y,
     *                    Output1x, Output1y, Output2x, Output2y,
     *                    DataType)

      implicit none

      real * 8 Input1x, Input1y
      real * 8 Input2x, Input2y
      real * 8 Output1x, Output1y
      real * 8 Output2x, Output2y

      character * 8 DataType

c  determine if input is in UTM or Lat/Long (Geographic)

      if ((abs(Input1x) .gt. 360.0) .or.
     *    (abs(Input1y) .gt. 360.0) .or.
     *    (abs(Input2x) .gt. 360.0) .or.
     *    (abs(Input2y) .gt. 360.0)) then

          DataType(1:3) = 'UTM'

          Output1x = Input1x
          Output1y = Input1y

          Output2x = Input2x
          Output2y = Input2y

c  assure that both points are referenced to the northern hemisphere
c  (UTM Y's south of the Equator have 10,000,000 added to them)

          if (Input1y .gt. 5000000.0)
     *        Output1y = Output1y - 10000000.0

          if (Input2y .gt. 5000000.0)
     *        Output2y = Output2y - 10000000.0

      else if ((abs(Input1x) .le.  90.0) .and.
     *         (abs(Input1y) .le. 360.0) .and.
     *         (abs(Input2x) .le.  90.0) .and.
     *         (abs(Input2y) .le. 360.0)) then

          DataType(1:8) = 'LAT/LONG'

c  flip lat/long x/y relationship to y/x

          Output1x = Input1y
          Output1y = Input1x

          Output2x = Input2y
          Output2y = Input2x

c  assure that both points are referenced to the same E/W hemisphere

          if (abs(Input1x - Input2x) .gt. 180.0) then

              if (Input1x .lt. Input2x) then
                  Output1x = Output1x + 360.0

              else if (Input1x .gt. Input2x) then
                  Output2x = Output2x + 360.0

              end if

          end if

      else
          DataType(1:7) = 'UNKNOWN'

          print *, 'Invalid postions of ',
     *             Input1x, ',', Input1y, ' and ',
     *             Input2x, ',', Input2y
          print *, '  used in call to TYPENAV subroutine.'
      end if

      return
      end
